/*
 Copyright (c) 2012 G. Bartsch

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.

 Credits go to:

 arduino-tvout Copyright (c) 2010 Myles Metzer 
 http://code.google.com/p/arduino-tvout/source/browse/TVout.cpp

 AVR Videogenerator, 40x25 Zeichen, nur 60% CPU Auslastung ! by Benedikt
 http://www.mikrocontroller.net/topic/53141#413260

 Simple VGA/Video Adapter von Ibragimov Maxim Rafikovich
 http://www.serasidis.gr/circuits/AVR_VGA/avr_vga.htm
*/

/*
 * Handy 2d gfx engine for tvgfx
 *
 * mostly based on arduino-tvout code.
 */

#ifndef HAVE_TVGFX_DRAW2D_H
#define HAVE_TVGFX_DRAW2D_H

#define TVGFX_WHITE 1
#define TVGFX_BLACK 0
#define TVGFX_TRANS -1
#define TVGFX_INVERT 3

void tvgfx_clr(void);

void tvgfx_set_pixel(int16_t x, int16_t y, char c) ;

char tvgfx_get_pixel(int16_t x, int16_t y) ;

/*
 * fast, bresenham based 2D gfx
 *
 * see http://free.pages.at/easyfilter/bresenham.html for details
 *
 */

void tvgfx_line(int16_t x0, int16_t y0, int16_t x1, int16_t y1, char c) ;

void tvgfx_row(int16_t y0, int16_t x0, int16_t x1, char c) ;

void tvgfx_column(int16_t x0, int16_t y0, int16_t y1, char c) ;

void tvgfx_rect(int16_t x0, int16_t y0, int16_t w, int16_t h, char c, char color_bg) ;

void tvgfx_circle(int16_t x0, int16_t y0, int16_t radius, char c, char color_bg) ;

void tvgfx_ellipse (int16_t x0, int16_t y0, int16_t xr, int16_t yr, char c, char color_bg) ;

/*
 * Advanced 2D engine
 *
 * see:
 *
 *   http://rosettacode.org/wiki/Raster_graphics_operations
 *   http://alienryderflex.com/polygon_fill/
 *   http://code-heaven.blogspot.de/2009/10/simple-c-program-for-scan-line-polygon.html
 *
 */

void tvgfx_linex(int16_t x0, int16_t y0, int16_t x1, int16_t y1, char c, uint8_t thickness);
void tvgfx_rectx(int16_t x0, int16_t y0, int16_t w, int16_t h, char c, char color_bg, uint8_t thickness);

// paths

void    tvgfx_set_thickness (uint8_t thickness);
uint8_t tvgfx_get_thickness (void);
void    tvgfx_set_foreground (uint8_t color);
uint8_t tvgfx_get_foreground (void);
void    tvgfx_set_background (uint8_t color);
uint8_t tvgfx_get_background (void);

void tvgfx_move_to (int16_t x, int16_t y);
void tvgfx_line_to (int16_t x, int16_t y);
void tvgfx_arc_to (int16_t x2, int16_t y2, char fa, char fs, int16_t rx, int16_t ry, int16_t phi);
void tvgfx_qbezier_to (int16_t x1, int16_t y1, int16_t x2, int16_t y2);
void tvgfx_cbezier_to (int16_t x1, int16_t y1, int16_t x2, int16_t y2, int16_t x3, int16_t y3);
void tvgfx_stroke (void);
void tvgfx_fill (void);

/*
 * floodfill (very slow)
 */

void tvgfx_flood (int16_t x, int16_t y);

/*
 * Bitmaps
 */

#ifdef TVGFX_FASTFONTS
void tvgfx_bitmap(int16_t x, int16_t y, const unsigned char *bmp, int16_t offset, uint8_t width, uint8_t lines) ;
#else
void tvgfx_bitmap(int16_t x, int16_t y, uint32_t bmp, int16_t offset, uint8_t width, uint8_t lines) ;
#endif

/*
 * Fonts and text
 */

#ifdef TVGFX_FASTFONTS

#define TVGFX_SETFONT(font) tvgfx_setfont(font)
void tvgfx_setfont (const unsigned char *f) ;

#else

#define TVGFX_SETFONT(font) tvgfx_setfont(FAR(font))
void tvgfx_setfont (uint32_t f) ;

#endif

uint8_t tvgfx_getfontheight(void);

int16_t tvgfx_textwidth(char *str);

uint8_t tvgfx_putc(int16_t x, int16_t y, unsigned char c);

int16_t tvgfx_puts (int16_t x, int16_t y, char *str);

void tvgfx_printf(int16_t x, int16_t y, const char *fmt, ...);


#endif


