/*
 Copyright (c) 2012, 2013 G. Bartsch

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.

 Credits go to:

 arduino-tvout Copyright (c) 2010 Myles Metzer 
 http://code.google.com/p/arduino-tvout/source/browse/TVout.cpp

 AVR Videogenerator, 40x25 Zeichen, nur 60% CPU Auslastung ! by Benedikt
 http://www.mikrocontroller.net/topic/53141#413260

 Simple VGA/Video Adapter von Ibragimov Maxim Rafikovich
 http://www.serasidis.gr/circuits/AVR_VGA/avr_vga.htm
*/


#ifndef HAVE_TVGFX_VIDEO_H
#define HAVE_TVGFX_VIDEO_H

#define NTSC

#define XSize       52   // bytes per scanline

#define XLineTimer  40   // what time to set up X line draw timer for
#define XLineDelay  73   // actual line output starts at XLineTimer+XLineDelay
                         // the larger the difference, the better robustness against jitter

#define HSync       75   // cpu cycles

#ifndef NTSC

// PAL

#define VLines      313  // total lines: visible lines + sync lines
#define YSize       256  // visible lines
#define BPSize      30   // VSync back porch in lines (Y offset)
#define VSync       4    // lines

#else

// NTSC

#define VLines      263  // total lines: visible lines + sync lines
#define YSize       226  // visible lines
#define BPSize      27   // VSync back porch in lines (Y offset)
#define VSync       4    // lines

#endif

/*
 * Init the composite video engine
 *
 * All video output is done in Timer1 interrupt which is also
 * set up and enabled here. 
 *
 */

void tvgfx_init(void) ;

/*
 * Global variables
 */

// frame buffer memory (XSize * YSize bytes):

extern volatile char image_mem[]; 

// frame counter - for benchmarking an VBlank-sync drawing
// (will be updated as soon as vsync interval begins so you
// may want to busy-wait for it to change

extern volatile uint16_t frame;

// Add hooks to be executed at each frame (vbi) / scanline (hbi)

void tvgfx_set_hbi_hook(void (*hbi_hook)(void));

void tvgfx_set_vbi_hook(void (*vbi_hook)(void));

#endif


